@@ -1,9 +1,9 @@
-use crate::{core::DateTime, Frequency, NWeekday, RRule, RRuleError, RRuleProperties, RRuleSet};
+use crate::{core::DateTime, Frequency, NWeekday, RRuleError, RRuleProperties, RRuleSet};
 use chrono::{Datelike, NaiveDate, TimeZone, Timelike, Weekday};
 use chrono_tz::{Tz, UTC};
 use lazy_static::lazy_static;
 use regex::Regex;
-use std::{convert::TryFrom, str::FromStr};
+use std::str::FromStr;
 
 use super::ParseError;
 
@@ -39,8 +39,7 @@ pub(crate) fn build_rruleset(s: &str) -> Result<RRuleSet, RRuleError> {
     };
 
     for rrule_prop in rrule_vals.iter() {
-        let parsed_opts = rrule_prop.clone().dt_start(dt_start);
-        let rrule = RRule::new(parsed_opts)?;
+        let rrule = rrule_prop.clone().build(dt_start)?;
         rset.rrule(rrule);
     }
 
@@ -49,8 +48,7 @@ pub(crate) fn build_rruleset(s: &str) -> Result<RRuleSet, RRuleError> {
     }
 
     for exrule_prop in exrule_vals.iter() {
-        let parsed_opts = exrule_prop.clone().dt_start(dt_start);
-        let exrule = RRule::new(parsed_opts)?;
+        let exrule = exrule_prop.clone().build(dt_start)?;
         rset.exrule(exrule);
     }
 
@@ -63,32 +61,27 @@ pub(crate) fn build_rruleset(s: &str) -> Result<RRuleSet, RRuleError> {
 
 /// Create an [`RRule`] from [`String`] if input is valid.
 ///
-/// If RRule contains invalid parts then [`RRuleError`] will be returned.
+/// If RRule contains invalid parts then [`ParseError`] will be returned.
 /// This should never panic, but it might be in odd cases.
 /// Please report if it does panic.
 pub(crate) fn parse_rrule_string_to_properties(input: &str) -> Result<RRuleProperties, ParseError> {
     let input = preprocess_rrule_string(input);
 
-    let ParsedInput {
-        mut rrule_vals,
-        dt_start,
-        ..
-    } = parse_input(&input)?;
+    let ParsedInput { mut rrule_vals, .. } = parse_input(&input)?;
 
     match rrule_vals.len() {
         0 => Err(ParseError::Generic("Invalid rrule string".into())),
         1 => {
-            let rrule_opts = rrule_vals.remove(0);
-            let parsed_opts = rrule_opts.dt_start(dt_start);
-            Ok(parsed_opts)
+            let rrule_properties = rrule_vals.remove(0);
+            Ok(rrule_properties)
         }
         _ => Err(ParseError::Generic(
-            "To many rrules, please use `RRuleSet` instead.".into(),
+            "Too many rrules, please use `RRuleSet` instead.".into(),
         )),
     }
 }
 
-/// Fill in some additional field in order to make iter work correctly.
+/// Fill in some additional fields in order to make iter work correctly.
 pub(crate) fn finalize_parsed_properties(
     mut properties: RRuleProperties,
     dt_start: &DateTime,
@@ -180,8 +173,11 @@ fn parse_timezone(tz: &str) -> Result<Tz, ParseError> {
     Tz::from_str(tz).map_err(|_err| ParseError::InvalidTimezone(tz.into()))
 }
 
-// TODO: errors should not be specific to DTSTART
-fn datestring_to_date(dt: &str, tz: &Option<Tz>, field: &str) -> Result<DateTime, ParseError> {
+pub(crate) fn datestring_to_date(
+    dt: &str,
+    tz: &Option<Tz>,
+    field: &str,
+) -> Result<DateTime, ParseError> {
     let bits = DATESTR_RE
         .captures(dt)
         .ok_or_else(|| ParseError::InvalidDateTime {
@@ -275,18 +271,14 @@ fn datestring_to_date(dt: &str, tz: &Option<Tz>, field: &str) -> Result<DateTime
             }
         }
     };
-    let datetime_with_timezone = if let Some(tz) = tz {
-        // Apply timezone from `TZID=` part (if any)
-        datetime.with_timezone(tz)
-    } else {
-        // If no timezone is give, set datetime as UTC
-        datetime.with_timezone(&UTC)
-    };
+
+    // Apply timezone from `TZID=` part (if any), else set datetime as UTC
+    let datetime_with_timezone = datetime.with_timezone(&tz.unwrap_or(UTC));
 
     Ok(datetime_with_timezone)
 }
 
-fn parse_dtstart(s: &str) -> Result<DateTime, ParseError> {
+pub(crate) fn parse_dtstart(s: &str) -> Result<DateTime, ParseError> {
     let caps = DTSTART_RE.captures(s).ok_or(ParseError::MissingStartDate)?;
 
     let tz: Option<Tz> = match caps.get(1) {
@@ -305,19 +297,6 @@ fn parse_dtstart(s: &str) -> Result<DateTime, ParseError> {
     datestring_to_date(dt_start_str, &tz, "DTSTART")
 }
 
-fn weekday_from_str(val: &str) -> Result<Weekday, String> {
-    match val {
-        "MO" => Ok(Weekday::Mon),
-        "TU" => Ok(Weekday::Tue),
-        "WE" => Ok(Weekday::Wed),
-        "TH" => Ok(Weekday::Thu),
-        "FR" => Ok(Weekday::Fri),
-        "SA" => Ok(Weekday::Sat),
-        "SU" => Ok(Weekday::Sun),
-        _ => Err(format!("Invalid weekday: {}", val)),
-    }
-}
-
 fn stringval_to_int<T: FromStr>(val: &str, err_msg: String) -> Result<T, ParseError> {
     if let Ok(val) = val.parse() {
         Ok(val)
@@ -347,13 +326,13 @@ fn stringval_to_intvec<T: FromStr + Ord + PartialEq + Copy, F: Fn(T) -> bool>(
     Ok(parsed_vals)
 }
 
-fn parse_rrule(line: &str, mut dt_start: Option<DateTime>) -> Result<RRuleProperties, ParseError> {
+fn parse_rrule(line: &str) -> Result<RRuleProperties, ParseError> {
     // Store all parts independently, so we can see if things are double set or missing.
     let mut freq = None;
     let mut interval = None;
     let mut count = None;
     let mut until = None;
-    let mut tz = None;
+    // let mut tz = None;
     // let mut dt_start = None;
     let mut week_start = None;
     let mut by_set_pos = None;
@@ -382,7 +361,7 @@ fn parse_rrule(line: &str, mut dt_start: Option<DateTime>) -> Result<RRuleProper
 
         match key.to_uppercase().as_str() {
             "FREQ" => {
-                let new_freq = Frequency::try_from(value)?;
+                let new_freq = Frequency::from_str(value)?;
                 if freq.is_none() {
                     freq = Some(new_freq)
                 } else {
@@ -420,12 +399,12 @@ fn parse_rrule(line: &str, mut dt_start: Option<DateTime>) -> Result<RRuleProper
                 }
             }
             "DTSTART" | "TZID" => {
-                // for backwards compatibility
-                let dtstart_opts = parse_dtstart(line)?;
-                tz = Some(dtstart_opts.timezone());
-                dt_start = Some(dtstart_opts);
+                // for backward compatibility
+                // let dtstart_opts = parse_dtstart(line)?;
+                // tz = Some(dtstart_opts.timezone());
+                // dt_start = Some(dtstart_opts);
             }
-            "WKST" => match weekday_from_str(value) {
+            "WKST" => match str_to_weekday(value) {
                 Ok(new_weekday) => {
                     if week_start.is_none() {
                         week_start = Some(new_weekday)
@@ -434,7 +413,7 @@ fn parse_rrule(line: &str, mut dt_start: Option<DateTime>) -> Result<RRuleProper
                     }
                 }
                 Err(e) => {
-                    return Err(ParseError::Generic(e));
+                    return Err(e);
                 }
             },
             "BYSETPOS" => {
@@ -556,7 +535,7 @@ fn parse_rrule(line: &str, mut dt_start: Option<DateTime>) -> Result<RRuleProper
         };
     }
 
-    // Check if manditory fields are set
+    // Check if mandatory fields are set
     Ok(RRuleProperties {
         freq: freq.ok_or_else(|| {
             ParseError::Generic(format!("Property `FREQ` was missing in `{}`", line))
@@ -565,11 +544,6 @@ fn parse_rrule(line: &str, mut dt_start: Option<DateTime>) -> Result<RRuleProper
         interval: interval.unwrap_or(1),
         count,
         until,
-        tz: tz.unwrap_or(UTC), // TODO check, I think this should be local timezone
-        dt_start: dt_start.unwrap_or_else(|| UTC.ymd(1970, 1, 1).and_hms(0, 0, 0)), // Unix Epoch
-        // dt_start: dt_start.ok_or_else(|| {
-        //     ParseError::Generic(format!("Property `DTSTART` was missing in `{}`", line))
-        // })?,
         week_start: week_start.unwrap_or(Weekday::Mon),
         by_set_pos: by_set_pos.unwrap_or_default(),
         by_month: by_month.unwrap_or_default(),
@@ -585,7 +559,7 @@ fn parse_rrule(line: &str, mut dt_start: Option<DateTime>) -> Result<RRuleProper
     })
 }
 
-fn str_to_weekday(d: &str) -> Result<Weekday, ParseError> {
+pub(crate) fn str_to_weekday(d: &str) -> Result<Weekday, ParseError> {
     let day = match &d.to_uppercase()[..] {
         "MO" => Weekday::Mon,
         "TU" => Weekday::Tue,
@@ -599,6 +573,18 @@ fn str_to_weekday(d: &str) -> Result<Weekday, ParseError> {
     Ok(day)
 }
 
+pub(crate) fn weekday_to_str(d: &Weekday) -> String {
+    match d {
+        Weekday::Mon => "MO".to_string(),
+        Weekday::Tue => "TU".to_string(),
+        Weekday::Wed => "WE".to_string(),
+        Weekday::Thu => "TH".to_string(),
+        Weekday::Fri => "FR".to_string(),
+        Weekday::Sat => "SA".to_string(),
+        Weekday::Sun => "SU".to_string(),
+    }
+}
+
 /// Parse the "BYWEEKDAY" and "BYDAY" values
 /// Example: `SU,MO,TU,WE,TH,FR` or `4MO` or `-1WE`
 /// > For example, within a MONTHLY rule, +1MO (or simply 1MO) represents the first Monday
@@ -658,11 +644,7 @@ fn parse_rule_line(rfc_string: &str) -> Result<Option<RRuleProperties>, ParseErr
         };
 
         match key {
-            "EXRULE" | "RRULE" => Ok(Some(parse_rrule(rfc_string, None)?)),
-            "DTSTART" => Ok(Some(RRuleProperties::new(
-                Frequency::Yearly, // TODO this value should not be a default
-                parse_dtstart(rfc_string)?,
-            ))),
+            "EXRULE" | "RRULE" => Ok(Some(parse_rrule(rfc_string)?)),
             _ => Err(ParseError::Generic(format!(
                 "Unsupported RFC prop {} in {}",
                 key, &rfc_string
@@ -670,7 +652,7 @@ fn parse_rule_line(rfc_string: &str) -> Result<Option<RRuleProperties>, ParseErr
         }
     } else {
         // If no header is set, we can parse it as `RRULE`
-        Ok(Some(parse_rrule(rfc_string, None)?))
+        Ok(Some(parse_rrule(rfc_string)?))
     }
 }
 
@@ -715,7 +697,7 @@ fn extract_name(line: String) -> LineName {
     }
 }
 
-fn parse_rule(rfc_string: &str) -> Result<RRuleProperties, ParseError> {
+pub(crate) fn parse_rule(rfc_string: &str) -> Result<RRuleProperties, ParseError> {
     let mut option = None;
     for line in rfc_string.split('\n') {
         let parsed_line = parse_rule_line(line)?;
@@ -724,7 +706,7 @@ fn parse_rule(rfc_string: &str) -> Result<RRuleProperties, ParseError> {
                 option = Some(parsed_line);
             } else {
                 return Err(ParseError::Generic(format!(
-                    "Found to many RRule lines in `{}`.",
+                    "Found too many RRule lines in `{}`.",
                     rfc_string
                 )));
             }
@@ -748,9 +730,6 @@ struct ParsedInput {
     exrule_vals: Vec<RRuleProperties>,
     exdate_vals: Vec<DateTime>,
     dt_start: DateTime,
-    // TODO: Why is this field never used?
-    #[allow(dead_code)]
-    tz: Tz,
 }
 
 fn parse_input(s: &str) -> Result<ParsedInput, ParseError> {
@@ -759,7 +738,6 @@ fn parse_input(s: &str) -> Result<ParsedInput, ParseError> {
     let mut exrule_vals = vec![];
     let mut exdate_vals = vec![];
     let dt_start = parse_dtstart(s)?;
-    let tz = dt_start.timezone();
 
     let lines: Vec<&str> = s.split('\n').collect();
     for line in &lines {
@@ -831,7 +809,6 @@ fn parse_input(s: &str) -> Result<ParsedInput, ParseError> {
 
     Ok(ParsedInput {
         dt_start,
-        tz,
         rrule_vals,
         rdate_vals,
         exrule_vals,
@@ -879,6 +856,7 @@ fn preprocess_rrule_string(s: &str) -> String {
 mod test {
     use super::*;
     use crate::DateFilter;
+    use crate::RRule;
 
     /// Print and compare 2 lists of dates and panic it they are not the same.
     fn check_occurrences(occurrences: Vec<DateTime>, expected: Vec<&str>) {
@@ -909,7 +887,7 @@ mod test {
         let tests = [
 "DTSTART:19970902T090000Z\nRRULE:FREQ=YEARLY;COUNT=3\n",
 "DTSTART:20120201T093000Z\nRRULE:FREQ=WEEKLY;INTERVAL=5;UNTIL=20130130T230000Z;BYDAY=MO,FR",
-"RRULE:UNTIL=19990404T110000Z;DTSTART;TZID=America/Denver:19990104T110000Z;FREQ=WEEKLY;BYDAY=TU,WE",
+"DTSTART;TZID=America/Denver:19990104T110000Z\nRRULE:UNTIL=19990404T110000Z;FREQ=WEEKLY;BYDAY=TU,WE",
 "DTSTART:20120201T120000Z\nRRULE:FREQ=DAILY;COUNT=5\nEXDATE;TZID=Europe/Berlin:20120202T130000Z,20120203T130000Z"
         ];
         for test_str in tests {
@@ -962,6 +940,8 @@ mod test {
         for test_case in &test_cases {
             let res = parse_rrule_string_to_properties(test_case);
             assert!(res.is_err());
+            let res = crate::parser::parse_dtstart(test_case);
+            assert!(res.is_err());
         }
     }
 
@@ -1084,7 +1064,7 @@ mod test {
         let res = parse_rrule_string_to_properties("FREQ=DAILY;COUNT=7");
         assert!(res.is_err());
         let err = res.unwrap_err();
-        assert_eq!(err, ParseError::MissingStartDate.into());
+        assert_eq!(err, ParseError::MissingStartDate);
     }
 
     #[test]
@@ -1196,10 +1176,7 @@ mod test {
             vec![NWeekday::new(None, Weekday::Mon)]
         );
         assert_eq!(rrule.get_properties().interval, 1);
-        assert_eq!(
-            rrule.get_properties().dt_start,
-            UTC.ymd(2021, 4, 5).and_hms(15, 0, 0)
-        );
+        assert_eq!(rrule.dt_start, UTC.ymd(2021, 4, 5).and_hms(15, 0, 0));
     }
 
     #[test]
@@ -1243,7 +1220,7 @@ mod test {
     #[test]
     fn parse_datetime_with_timezone() {
         let rrule: RRule =
-            "DTSTART;TZID=America/New_York:20120201T023000Z\nFREQ=DAILY;INTERVAL=1;COUNT=2"
+            "DTSTART;TZID=America/New_York:20120201T023000Z\nRRULE:FREQ=DAILY;INTERVAL=1;COUNT=2"
                 .parse()
                 .expect("RRule could not be parsed");
 
@@ -1308,7 +1285,7 @@ mod test {
     #[test]
     fn parse_datetime_errors_invalid_tzid_timezone() {
         let res = RRule::from_str(
-            "DTSTART;TZID=America/Everywhere:20120251T023000Z\nFREQ=DAILY;INTERVAL=1;COUNT=2",
+            "DTSTART;TZID=America/Everywhere:20120251T023000Z\nRRULE:FREQ=DAILY;INTERVAL=1;COUNT=2",
         );
         assert!(res.is_err());
         let err = res.unwrap_err();
